/*jslint browser: true*/
/*global define*/
define('de_epages/dhlintraship/ui/packstation', [
    'jquery',
    'ep',
    'google/maps',
    '$dict!../dictionary',
    '$tmpl!./packstation',
    '$tmpl!./packstation-row',
    '$tmpl!./packstation-popup',
    'jquery/ui/widget',
    'jquery/metaparse',
    'ep/ajax',
    'ep/fn/busy'
], function ($, ep, maps, epDict, tmpl, rowTmpl, popupTmpl) {

    'use strict';

    $.widget('ui.dhlintrashipUiPackstation', {
        options: {
            zipcode: '10178', // center of Berlin zipcode
            packstationId: null
        },

        _create: function () {
            var rendered = tmpl(this.options),
                tmplElements = rendered.tmplItem('elements');

            rendered
                .appendTo(this.element)
                .find('.ep-js')
                .metaparse();

            $.extend(this, tmplElements);

            this._update();

        },

        _update: function () {
            var self = this, // required to distinguish from internal this in "done"
                dhlPackstation = $('#DHLPackstation'),
                packstationElement = this.options.packstationSelector;
            dhlPackstation.busy('show');
            ep.ajax({ // AJAX request
                data: {
                    ObjectID: this.options.packstationId,
                    ViewAction: 'JSONViewPackstations',
                    Street: this.options.street,
                    StreetNo: this.options.streetNo,
                    City: this.options.city,
                    ZipCode: this.options.zipcode
                },
                cache: false,
                type: "POST",
                dataType: "json"
            }).done(function (data) {
            	dhlPackstation.busy('hide');
                self.packstationData = data;
                self._updateMap(dhlPackstation);
                self._updateLocationTable();

            }).fail(function (errJSON) {
                dhlPackstation.busy('hide');
                dhlPackstation.append('<p>' + epDict.translate('{PackstationServiceError}') + '</p>');
            });
        },

        _updateMap: function (dhlPackstation) {
            var self = this,
                enablePackstationP = $('#EnableDHLIntrashipPackstation').parent(),
                firstpackstation = this.packstationData[0];

            if ( firstpackstation ) {
                enablePackstationP.removeClass('HideElementSoft');
            } else {
                dhlPackstation.append('<p>' + epDict.translate('{NoPackstations}') + '</p>');
            }

            var mapOptions = {
                center: new maps.LatLng(firstpackstation.latitude, firstpackstation.longitude),
                zoom: 11,
                mapTypeId: maps.MapTypeId.ROADMAP
            };

            //initialize map
            var googleMap = new maps.Map(this.map.get(0), mapOptions);

            $.each(this.packstationData, function (i, data) {
                var distance = epDict.translate('{Distance}') + ': ' + data.distance,
                    myLatLng = new maps.LatLng(data.latitude, data.longitude),
                    packstation = new maps.Marker({
                        position: myLatLng,
                        map: googleMap,
                        title: data.name
                    }),
                    addrDataPopup = [{
                        nameP: data.name,
                        addressP: data.address,
                        zipP: data.zipcode,
                        cityP: data.city,
                        districtP: data.district,
                        distanceP: distance,
                        packstationIdP: data.packstationId
                    }],
                    contentString = popupTmpl(addrDataPopup),
                    infowindow = new maps.InfoWindow({
                        content: contentString.html()
                    });

                maps.event.addListener(packstation, 'click', function () {
                    // Close last open info window in map
                    if (self.lastInfowindow) {
                        self.lastInfowindow.close();
                    }
                    // Set current info window as last one
                    self.lastInfowindow = infowindow;
                    // Open info window
                    infowindow.open(googleMap, packstation);
                    // Select the radio button of the related info dialog
                    self._selectLocation(i);
                });
            });
        },

        _updateLocationTable: function () {
            $.each(this.packstationData, function (i, data) {
                data.distance = data.distance;
            });
            this.locations.empty()
                .append(rowTmpl({
                    addresses: this.packstationData,
                    packstationsNearby: epDict.translate('{PackstationsNearby}'),
                    otherPackstation: epDict.translate('{OtherPackstation}'),
                    packstationNumber: epDict.translate('{PackstationNumber}'),
                    zipcode: epDict.translate('{Zipcode}'),
                    city: epDict.translate('{City}')
                }))
                .find('.ep-js')
                .metaparse();

            this._selectLocation(0);
        },

        _selectLocation: function (i) {
            this.locations.find('.location_' + i)
                .prop('checked', 'checked');
        },

        destroy: function () {
            this.element.empty();

            this._superApply(arguments);
        }
    });
    return de_epages;
});