/*global define,window*/
define([
    'testsuite/qunit',
    'jquery',
    'ep',
    'testsuite/qunit-fixture!./ipaymentbillsafefields-test',
    'de_epages/ipayment/ui/ipaymentbillsafefields',
    'ep/ui/datepicker'
], function (QUnit, $, ep, fixture, ipaymentbillsafefields) {

    'use strict';
    ep.ajax = function () {
        var result = $.Deferred();
        return result;
    };

    QUnit.module('de_epages/ipayment/ui/ipaymentbillsafefields', {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.test('Check that Datepicker gets initialized with current date', 1, function () {

        $("#dp_birthday").uiDatepicker({
            'type': 'date',
            'required': true,
            'format': 'd',
            'time': false
        });
        var pickedDate = $("#dp_birthday").uiDatepicker('getDate'),
            currentDate = new Date();

        QUnit.strictEqual(
            pickedDate.getFullYear() + '/' + (pickedDate.getMonth() + 1) + '/' + pickedDate.getDate(),
            currentDate.getFullYear() + '/' + (currentDate.getMonth() + 1) + '/' + currentDate.getDate(),
            'Datepicker default date and current date are the same: ' + pickedDate.getDate() + '/' + (pickedDate.getMonth() + 1) + '/' + pickedDate.getFullYear()
        );
    });

    QUnit.test('Hidden gender and picker fields values are not set and Datepicker gets initialized. We expect a false from ipaymentbillsafefields', 1, function () {

        $("#dp_birthday").uiDatepicker({
            'type': 'date',
            'required': true,
            'format': 'd',
            'time': false
        });

        QUnit.strictEqual(
            ipaymentbillsafefields([], 1, 'invoice', $('#addr_birthday'), $('#dp_birthday'), $('#male_gender'), $('#female_gender')),
            false,
            'ipaymentbillsafefields function returns false on submit'
        );
    });

    QUnit.test('Hidden gender field value is not set, hidden birthday field value is set and Datepicker doesn\' exist. We expect a false from ipaymentbillsafefields', 3, function () {
        $('#dp_birthday').remove();
        $('#addr_birthday').val('05/11/1980');

        QUnit.strictEqual(
            $('#dp_birthday').length,
            0,
            'datepicker length is zero because it doesn\'t exist'
        );
        QUnit.strictEqual(
            $('#addr_birthday').val(),
            '05/11/1980',
            'hidden birthday field value is set to 05/11/1980'
        );
        QUnit.strictEqual(
            ipaymentbillsafefields([], 1, 'invoice', $('#addr_birthday'), $('#dp_birthday'), $('#male_gender'), $('#female_gender')),
            false,
            'ipaymentbillsafefields function returns false on submit'
        );
    });

    QUnit.test('Hidden gender field doesn\'t exist, hidden birthday field value is set and visual malegender radio button is checked. We expect a true from ipaymentbillsafefields', 3, function () {
        $('#male_gender').prop('checked', true);

        QUnit.strictEqual(
            $('#male_gender').is(':checked'),
            true,
            'Visual male gender radio button is being checked'
        );

        $('#dp_birthday').remove();
        $('#addr_birthday').val('05/11/1980');
        QUnit.strictEqual(
            $('#addr_birthday').val(),
            '05/11/1980',
            'hidden birthday field value is set to 05/11/1980'
        );
        QUnit.strictEqual(
            ipaymentbillsafefields([], 1, 'invoice', $('#addr_birthday'), $('#dp_birthday'), $('#male_gender'), $('#female_gender')),
            true,
            'ipaymentbillsafefields function returns true on submit'
        );
    });
});